﻿using Autofac;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Threading.Tasks;

namespace Wms.Screen.Api
{
    public class AutofacModuleRegister : Autofac.Module
    {
        //重写Autofac管道Load方法，在这里注册注入
        protected override void Load(ContainerBuilder builder)
        {
            builder.RegisterAssemblyTypes(GetAssemblyByName("Wms.Screen.Service")).Where(a => a.Name.EndsWith("Service")).AsImplementedInterfaces().PropertiesAutowired();
            builder.RegisterAssemblyTypes(GetAssemblyByName("Wms.Screen.DataService")).Where(a => a.Name.EndsWith("Service")).AsImplementedInterfaces().PropertiesAutowired();

            //单一注册
            //builder.RegisterType<UserService>().Named<IUserService>(typeof(UserService).Name);

            //builder.RegisterType<NlogUtil>().Named<ILogUtil>(typeof(NlogUtil).Name);
            //builder.RegisterType<DataToExpressionService>().Named<IDataToExpressionService>(typeof(DataToExpressionService).Name);
            //builder.RegisterType<Log4NetUtil>().Named<ILogUtil>(typeof(Log4NetUtil).Name);


            //builder.RegisterType<DataRuleFactory>().PropertiesAutowired().InstancePerLifetimeScope();
            //在控制器中使用属性依赖注入，其中注入属性必须标注为public
            var controllersTypesInAssembly = typeof(Startup).Assembly.GetExportedTypes()
            .Where(type => typeof(Microsoft.AspNetCore.Mvc.ControllerBase).IsAssignableFrom(type)).ToArray();
            builder.RegisterTypes(controllersTypesInAssembly).PropertiesAutowired();

        }
        /// <summary>
          /// 根据程序集名称获取程序集
          /// </summary>
          /// <param name="AssemblyName">程序集名称</param>
        public static Assembly GetAssemblyByName(String AssemblyName)
        {
            return Assembly.Load(AssemblyName);
        }
    }
}
