using Autofac;
using AutoMapper;
using Bozhon.Wms.Api.Extensions;
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Hosting;
using Microsoft.OpenApi.Models;
using SqlSugar;
using System;
using System.IO;
using System.Reflection;
using Wms.Screen.Api.Extensions;
using Wms.Screen.Service.Mapper;
using Wms.Screen.SqlSugar;

namespace Wms.Screen.Api
{
    public class Startup
    {
        public Startup(IConfiguration configuration)
        {
            Configuration = configuration;
        }

        public IConfiguration Configuration { get; }

        public void ConfigureServices(IServiceCollection services)
        {
            services.AddCors(options =>
            {
                options.AddPolicy(name: "cors",
                    builder => 
                    builder.AllowAnyOrigin()
                    .AllowAnyHeader()
                    .WithMethods("GET", "POST", "HEAD", "PUT", "DELETE", "OPTIONS")
                    );
            });
            // עSwagger
            services.AddSwaggerGen(c =>
            {
                // ĵϢ
                c.SwaggerDoc("v1", new OpenApiInfo
                {
                    Title = "ӿĵ",
                    Version = "v1.0.0",
                    Description = "WMS ӿ",
                });
                // ʹ÷ȡxmlļļ·
                var xmlFile = $"{Assembly.GetExecutingAssembly().GetName().Name}.xml";
                var xmlPath = Path.Combine(AppContext.BaseDirectory, xmlFile);
                // xmlע. ÷ڶÿעͣĬΪfalse.
                c.IncludeXmlComments(xmlPath, true);
            });

            var sqlSugarConfig = SqlSugarConfig.GetConnectionString(Configuration);
            services.AddSqlSugarClient<SqlSugarClient>(config =>
            {
                config.ConnectionString = sqlSugarConfig.Item2;
                config.DbType = sqlSugarConfig.Item1;
                config.IsAutoCloseConnection = true;
                config.InitKeyType = InitKeyType.Attribute;
                //config.IsShardSameThread = true;
            });


            services.AddAutoMapper(typeof(MyProfile));
            services.AddCSRedisCache(Configuration);

            services.AddHostedService<HostedServiceExtension>();
            services.AddControllers();
        }

        // This method gets called by the runtime. Use this method to configure the HTTP request pipeline.
        public void Configure(IApplicationBuilder app, IWebHostEnvironment env)
        {
            if (env.IsDevelopment())
            {
                app.UseDeveloperExceptionPage();
            }

            app.UseHttpsRedirection();
            // Swaggerм
            app.UseSwagger();
            // SwaggerUI
            app.UseSwaggerUI(c =>
            {
                c.SwaggerEndpoint("/swagger/v1/swagger.json", "1.0.0");
                c.RoutePrefix = string.Empty;
            });

            app.UseRouting();
            app.UseCors("cors");
            app.UseAuthorization();

            //DefaultFilesOptions defaultFilesOptions = new DefaultFilesOptions();
            //defaultFilesOptions.DefaultFileNames.Clear();
            //defaultFilesOptions.DefaultFileNames.Add("index.html");
            //app.UseDefaultFiles(defaultFilesOptions);

            //StaticFileOptions staticFileOptions = new StaticFileOptions();
            //staticFileOptions.FileProvider = new Microsoft.Extensions.FileProviders.PhysicalFileProvider(@"E:\code\screen\Wms.Screen.Api\Wms.Screen.Api\bin\Debug\netcoreapp3.1\publish\wwwroot\");
            //staticFileOptions.FileProvider = new Microsoft.Extensions.FileProviders.PhysicalFileProvider(@"D:\IIS\KANBAN\wwwroot\");
            //staticFileOptions.FileProvider = new Microsoft.Extensions.FileProviders.PhysicalFileProvider(@"D:\BZ\KANBAN\wwwroot\");
            //staticFileOptions.FileProvider = new Microsoft.Extensions.FileProviders.PhysicalFileProvider(@"C:\iis\KOHLER_KANBAN\wwwroot\");
            //app.UseStaticFiles(staticFileOptions);//̬ļ


            app.UseEndpoints(endpoints =>
            {
                endpoints.MapControllers();
            });
        }
        public void ConfigureContainer(ContainerBuilder builder)
        {
            //עϵ
            builder.RegisterModule(new AutofacModuleRegister());
        }
    }
}
