﻿using SqlSugar;
using System;
using System.Collections.Generic;
using System.Text;
using Wms.Screen.DataService.Interface;
using Wms.Screen.Dto.ZhongTian.Request;
using Wms.Screen.Dto.ZhongTian.Response;
using Wms.Screen.Dto.ZhongTian;
using Wms.Screen.SqlSugar.ZhongTian;
using System.Linq;
using Newtonsoft.Json;
using Wms.Screen.SqlSugar.ZhongTian.CP;
using BillBomsetinfo = wms.sqlsugar.model.fj.BillBomsetinfo;
using BillInvnow = wms.sqlsugar.model.cp.BillInvnow;
using CSRedis;

namespace Wms.Screen.DataService.Impl
{
    public class ZhongTianCpDataService : IZhongTianCpDataService
    {
        private SqlSugarClient _db;
        public ZhongTianCpDataService(SqlSugarClient db)
        {
            _db = db;
        }

        public List<WCS_TaskOld> GetCpEquips(GetEquipsRequest reqEntity)
        {
            var list = _db.Queryable<WCS_TaskOld>().Where(p => p.Status != TaskStatus.Finish && p.Status != TaskStatus.Cancel &&
            (p.Type == TaskType.OutDepot || p.Type == TaskType.EnterDepot) && !string.IsNullOrEmpty(p.Tunnel))
                .SplitTable(tabs => tabs.Take(2))
                .ToList();
            return list;
        }
        public List<TaskInfoDto> GetCpRunWcsTaskInfo(GetWcsTaskInfoRequest request)
        {
            var list = _db.Queryable<WCS_TaskOld>().SplitTable(p => p.Take(2))
                .LeftJoin<BillInvnow>((taskold, now) => taskold.BarCode == now.ContGrpBarCode)
                .Where(taskold => taskold.Status < TaskStatus.Finish)
                .WhereIF(request.TaskType > 0, (taskold, now) => taskold.Type == request.TaskType)
                .WhereIF(request.TaskTypelist != null && request.TaskTypelist.Any(), (taskold, now) => request.TaskTypelist.Contains(taskold.Type))
                .OrderBy((taskold, now) => taskold.AddTime, OrderByType.Desc)
                .Select((taskold, now) => new TaskInfoDto
                {
                    TaskCode = taskold.ID.ToString(),
                    TaskState = taskold.Status == TaskStatus.NewBuild ? "新建" : "执行中",
                    TaskType = taskold.Type == TaskType.EnterDepot ? "入库" : taskold.Type == TaskType.OutDepot ? "出库" : "移动",
                    HWBarCode = taskold.BarCode,
                    HWSpec = now.HWTypeCode,
                    MatCode=now.MatCode,
                    MatName=now.MatName,
                    BeginPosition = taskold.AddrFrom,
                    EndPosition = taskold.AddrTo,
                    CreatedTime = taskold.AddTime,
                    Tunnel = taskold.Tunnel,
                    AddTime = taskold.AddTime.ToString("HH:mm:ss"),
                    DocCode = taskold.TaskGroupKey
                }).Distinct().ToList();
            return list;
        }
        public List<ZTBillDto> GetCpBillList(GetWorkPlanBillListRequest reqEntity)
        {
            var list = _db.Queryable<BillDocsinfo, BillInvnow, BaseMatinfo, BaseWarehouse>
             ((doc, barcode, material, house) => new object[] {
                    JoinType.Left, doc.DocsNo == barcode.InDocsNo,
                    JoinType.Left, barcode.MatId == material.Id,
                    JoinType.Left, doc.WarehouseId == house.Id
                 })
              .Where((doc, barcode, material, house) => doc.IsStop == 1)
              .WhereIF(!string.IsNullOrEmpty(reqEntity.WarehouseDocumentType), (doc, barcode, material, house) => doc.TypeNum == int.Parse(reqEntity.WarehouseDocumentType))
              .WhereIF(reqEntity.WarehouseDocumentTypeList != null && reqEntity.WarehouseDocumentTypeList.Any(), (doc, barcode, material, house) => reqEntity.WarehouseDocumentTypeList.Contains(doc.TypeNum.ToString()))
              .WhereIF(reqEntity.WarehouseDocumentStateList != null && reqEntity.WarehouseDocumentStateList.Any(), (doc, barcode, material, house) => reqEntity.WarehouseDocumentStateList.Contains(doc.StateNum.ToString()))
              .WhereIF(reqEntity.WarehouseCodeList.Any(), (doc, barcode, material, house) => reqEntity.WarehouseCodeList.Contains(house.Code))
              .Select((doc, barcode, material, house) => new ZTBillDto
              {
                  OutDocumentCode = doc.DocsNo,
                  MaterialName = material.Name,
                  MaterialCode = material.Code,
                  StockQuantity = 0,
                  TypeCode = doc.TypeNum.ToString(),
                  StateCode = doc.StateNum == DocState.DocState_Create.GetHashCode() ? "新建" : "执行中"
              }).Distinct().ToList();
            return list;
        }
        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public List<ZtLocationUsageReportViewDto> GetCpLocationUsageReportList()
        {
            int _sort = 1;
            var list = _db.Queryable<BaseWarehouse, BaseWarecell, BillInvnow>((wareHouse, location, groupStock) => new object[]
                   {
                   JoinType.Left,wareHouse.Id == location.WarehouseId,
                   JoinType.Left,location.ContGrpId == groupStock.ContGrpId && groupStock.InvStateCode == InvEcecState.InvEcecState_In.ToString()
                   })
                .Where((wareHouse, location, groupStock) => location.TypeNum == LocationType.LocationType_StorageLocation.GetHashCode())
                .GroupBy((wareHouse, location, groupStock) => new
                {
                    location.IsStop,
                    location.Tunnel,
                    location.StateNum,
                    wareHouse.Id,
                    wareHouse.Name,
                    location.ContGrpId,
                    location.Size
                }).Select((wareHouse, location, groupStock) => new
                {
                    Status = location.IsStop,
                    Tunnel = location.Tunnel,
                    LocationLock = location.StateNum,
                    WareHouseId = wareHouse.Id,
                    Name = wareHouse.Name,
                    GroupStockType = location.ContGrpId,
                    Size = location.Size,
                    Total = SqlFunc.AggregateDistinctCount(location.Id)
                }).ToList();

            //基表：巷道和仓库名称为维度查找总货位
            var baseTemp = list.GroupBy(s => new { s.Tunnel, s.WareHouseId, s.Name }).Select(m => new { Name = m.Key.Name, Tunnel = m.Key.Tunnel, Total = m.Sum(n => n.Total) }).ToList();
            //关联表：巷道和仓库名称为维度，查找可用货位：未停用.已锁定也算有效货位
            var useLocation = list.Where(s => s.Status == 1).GroupBy(s => new { s.Tunnel, s.WareHouseId, s.Name }).Select(m => new { Name = m.Key.Name, Tunnel = m.Key.Tunnel, Total = m.Sum(n => n.Total) }).ToList(); //&& s.LocationLock == DictionaryConst.LocationLockNoneCode
            //关联表：巷道和仓库名称为维度，查找锁定货位
            var lockLocation = list.Where(s => s.Status == 1 && s.LocationLock != LocationLock.LocationLock_None.GetHashCode()).GroupBy(s => new { s.Tunnel, s.WareHouseId, s.Name }).Select(m => new { Name = m.Key.Name, Tunnel = m.Key.Tunnel, Total = m.Sum(n => n.Total) }).ToList();
            //关联表：巷道和仓库名称为维度，查找停用货位
            var stopLocation = list.Where(s => s.Status == 0).GroupBy(s => new { s.Tunnel, s.WareHouseId, s.Name }).Select(m => new { Name = m.Key.Name, Tunnel = m.Key.Tunnel, Total = m.Sum(n => n.Total) }).ToList();
            //关联表：巷道和仓库名称为维度，查找物料货位
            var materialLocation = list.Where(s => s.Status == 1 && s.GroupStockType == 2 && s.LocationLock == LocationLock.LocationLock_None.GetHashCode()).GroupBy(s => new { s.Tunnel, s.WareHouseId, s.Name }).Select(m => new { Name = m.Key.Name, Tunnel = m.Key.Tunnel, Total = m.Sum(n => n.Total) }).ToList();
            //关联表：巷道和仓库名称为维度，查找容器货位
            var containerLocation = list.Where(s => s.Status == 1 && s.GroupStockType == 1 && s.LocationLock == LocationLock.LocationLock_None.GetHashCode()).GroupBy(s => new { s.Tunnel, s.WareHouseId, s.Name }).Select(m => new { Name = m.Key.Name, Tunnel = m.Key.Tunnel, Total = m.Sum(n => n.Total) }).ToList();
            //关联表：巷道和仓库名称为维度，查找小货位数量
            var littleSizeLocation = list.Where(s => s.Status == 1 && s.Size == 1 && s.LocationLock == LocationLock.LocationLock_None.GetHashCode() && s.GroupStockType != 2 && s.GroupStockType != 1).GroupBy(s => new { s.Tunnel, s.WareHouseId, s.Name }).Select(m => new { Name = m.Key.Name, Tunnel = m.Key.Tunnel, Total = m.Sum(n => n.Total) }).ToList();

            var locationUsageReportViewDtos = from basetemp in baseTemp
                                              join uselocation in useLocation.DefaultIfEmpty() on new { Tunnel = basetemp.Tunnel, Name = basetemp.Name } equals new { Tunnel = uselocation.Tunnel, Name = uselocation.Name } into useinfo
                                              join locklocation in lockLocation.DefaultIfEmpty() on new { Tunnel = basetemp.Tunnel, Name = basetemp.Name } equals new { Tunnel = locklocation.Tunnel, Name = locklocation.Name } into lockinfo
                                              join stoplocation in stopLocation.DefaultIfEmpty() on new { Tunnel = basetemp.Tunnel, Name = basetemp.Name } equals new { Tunnel = stoplocation.Tunnel, Name = stoplocation.Name } into stopinfo
                                              join materiallocation in materialLocation.DefaultIfEmpty() on new { Tunnel = basetemp.Tunnel, Name = basetemp.Name } equals new { Tunnel = materiallocation.Tunnel, Name = materiallocation.Name } into materialinfo
                                              join containerlocation in containerLocation.DefaultIfEmpty() on new { Tunnel = basetemp.Tunnel, Name = basetemp.Name } equals new { Tunnel = containerlocation.Tunnel, Name = containerlocation.Name } into containerinfo
                                              join littlesizelocation in littleSizeLocation.DefaultIfEmpty() on new { Tunnel = basetemp.Tunnel, Name = basetemp.Name } equals new { Tunnel = littlesizelocation.Tunnel, Name = littlesizelocation.Name } into littlesizeinfo
                                              orderby basetemp.Name + basetemp.Tunnel
                                              select new ZtLocationUsageReportViewDto()
                                              {
                                                  Sort = _sort++,
                                                  Tunnel = basetemp.Tunnel.ToString(),
                                                  WarehouseName = basetemp.Name,
                                                  AllLocationTotal = basetemp.Total,
                                                  CanUseLocation = useinfo.FirstOrDefault() == null ? 0 : useinfo.First().Total,
                                                  LockLocation = lockinfo.FirstOrDefault() == null ? 0 : lockinfo.First().Total,
                                                  StopLocation = stopinfo.FirstOrDefault() == null ? 0 : stopinfo.First().Total,
                                                  MaterilLocation = materialinfo.FirstOrDefault() == null ? 0 : materialinfo.First().Total,
                                                  ContainLocation = containerinfo.FirstOrDefault() == null ? 0 : containerinfo.First().Total,
                                                  LittleSizeLocation = littlesizeinfo.FirstOrDefault() == null ? 0 : littlesizeinfo.First().Total,
                                                  InStates = "启用",
                                                  OutStates = "启用"
                                              };
            return locationUsageReportViewDtos.OrderBy(p => p.WarehouseName).ToList();
        }
        
        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public List<BillInvnow> GetTrayProductList()
        {
            return _db.Queryable<BillInvnow>().Where(p => p.InvStateCode == InvEcecState.InvEcecState_In.ToString()).ToList();
        }
        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public List<BaseWarecell> GetLocationList()
        {
            return _db.Queryable<BaseWarecell>().ToList();
        }

        /// <summary>
        ///  获取产品信息
        /// </summary>
        /// <param name="taskNumber"> 产品对应的任务号 </param>
        /// <returns></returns>
        public DeliveryDetails GetProductInfoList(List<ProductInfoListRequset> taskInfos)
        {
            var deliveryDetails = new DeliveryDetails();

            try
            {
                // 获取待发货单数和待发货箱数
                var delivery = _db.Queryable<cpBillInvnow>()
                    .With(SqlWith.NoLock)
                    .Where(x => x.SalesDoc != null)
                    .Distinct()
                    .ToList();

                deliveryDetails.ShippingOrderQuantity = delivery.Select(x => x.SalesDoc).Distinct().Count();
                deliveryDetails.ShippingBoxQuantity = delivery.Count;

                // 获取任务信息及其相关的 ProductInfo 列表
                var taskIds = taskInfos.Select(x => x.TaskId).ToList();
                var taskList = _db.Queryable<WCS_TaskOld>()
                    .With(SqlWith.NoLock)
                    .Where(x => taskIds.Contains(x.ID))
                    .SplitTable(x => x.Take(2))
                    .ToList();

                var taskInfoMap = taskInfos.ToDictionary(t => t.TaskId, t => t.DeviceId);
                var productInfoList = taskList.Select(x => new ProductInfo
                {
                    BoxBarCode = x.TaskGroupKey,
                    TaskId = x.ID.ToString(),
                    DeviceId = taskInfoMap.TryGetValue(x.ID, out var deviceId) ? deviceId : string.Empty,
                    Winding = x.InvBarCode
                }).ToList();

                // 获取所有流水信息和装箱信息一次性查询，避免在循环内多次查询
                var boxBarCodes = productInfoList.Select(p => p.BoxBarCode).Distinct().ToList();
                var flowData = _db.Queryable<cpBillInvflow>()
                    .With(SqlWith.NoLock)
                    .Where(x => boxBarCodes.Contains(x.BoxBarCode))
                    .OrderByDescending(x => x.AddTime)
                    .ToList()
                    .GroupBy(f => f.BoxBarCode)
                    .ToDictionary(g => g.Key, g => g.First());

                var boxItemData = _db.Queryable<cpBoxItems>()
                    .With(SqlWith.NoLock)
                    .Where(x => boxBarCodes.Contains(x.HuNr))
                    .OrderByDescending(x => x.AddTime)
                    .ToList()
                    .GroupBy(b => b.HuNr)
                    .ToDictionary(g => g.Key, g => g.First());

                // 将数据填充到 productInfoList 中
                foreach (var productInfo in productInfoList)
                {
                    if (flowData.TryGetValue(productInfo.BoxBarCode, out var initFlow))
                    {
                        productInfo.MatName = initFlow.MatName;
                        productInfo.SalesDoc = initFlow.SalesDoc;
                    }

                    if (boxItemData.TryGetValue(productInfo.BoxBarCode, out var boxItem) && (boxItem.Winding.Contains("L") || boxItem.Winding.Contains("R")))
                        productInfo.Winding = boxItem.Winding;

                    // 设备 ID 分类分组并设定索引值
                    switch (productInfo.DeviceId)
                    {
                        case "8255":
                            productInfo.Index = 3;
                            deliveryDetails.LProductInfos.Add(productInfo);
                            break;
                        case "8256":
                            productInfo.Index = 2;
                            deliveryDetails.LProductInfos.Add(productInfo);
                            break;
                        case "8257":
                            productInfo.Index = 1;
                            deliveryDetails.LProductInfos.Add(productInfo);
                            break;
                        case "8263":
                            productInfo.Index = 1;
                            deliveryDetails.RProductInfos.Add(productInfo);
                            break;
                        case "8264":
                            productInfo.Index = 2;
                            deliveryDetails.RProductInfos.Add(productInfo);
                            break;
                        case "8265":
                            productInfo.Index = 3;
                            deliveryDetails.RProductInfos.Add(productInfo);
                            break;
                    }
                }

                // 按 Index 排序
                deliveryDetails.LProductInfos = deliveryDetails.LProductInfos.OrderBy(p => p.Index).ToList();
                deliveryDetails.RProductInfos = deliveryDetails.RProductInfos.OrderBy(p => p.Index).ToList();
            }
            catch (Exception e)
            {
                deliveryDetails.Value = $"{e.Message}:{e.StackTrace}";
            }

            return deliveryDetails;


        }
    }
}
