﻿using System;
using System.Collections.Generic;
using System.Text;

namespace Wms.Screen.Dto
{
    public class Result
    {
        public const int Success = 200;
        public const int Error = 500;

        public int StatusCode { get; set; }

        public string Message { get; set; }

        public string Exception { get; set; }

        public Result()
        {
            this.StatusCode = Success;
            this.Message = "操作成功";
        }
    }

    public class Result<T> : Result
    {
        public T ReturnValue { get; set; }

        public Result()
        {
        }

        public Result(T result)
            : base()
        {
            this.ReturnValue = result;
        }
    }

    public class PageResult<T> : Result
    {
        public int PageIndex { get; set; }

        public int PageSize { get; set; }

        public int PageCount
        {
            get
            {
                return (int)Math.Ceiling((decimal)this.TotalCount / PageSize);
            }
            set { }
        }

        public int TotalCount { get; set; }

        public List<T> ReturnValue { get; set; }
        public decimal SumQuantity { get; set; }


        public PageResult()
        {
            this.PageIndex = 1;
            this.PageSize = 10;
            this.PageCount = 0;
            this.TotalCount = 0;
            this.ReturnValue = new List<T>();
        }
        //public List<GetReportStaticRes> ReportStaticRes { get; set; }

        public List<ItemGroup> ItemGroupSum { get; set; }
    }

    public class ItemGroup
    {
        public decimal qty { get; set; }
        public string UnitName { get; set; }
    }
    public class ItemVal
    {
        public decimal Value { get; set; }
        public string Name { get; set; }
    }

    public class ResultContainQuantity : Result
    {
        public int MaterialQuantity { get; set; }
    }

    /// <summary>
    /// 分页查询请求体
    /// </summary>
    /// <typeparam name="T"></typeparam>
    public class PageRequest<T> where T : class
    {
        /// <summary>
        /// 当前页
        /// </summary>
        public int PageIndex { get; set; } = 10;

        /// <summary>
        /// 每页记录数
        /// </summary>
        public int PageSize { get; set; } = 1;

        /// <summary>
        /// 请求参数
        /// </summary>
        public T ConditionRequest { get; set; }

        /// <summary>
        /// 查询字段
        /// </summary>
        public string SqlFields { get; set; }

        /// <summary>
        /// 获取总记录数sql
        /// </summary>
        public string SqlRecordCount { get; set; }

        /// <summary>
        /// 排序字段
        /// </summary>
        public string OrderByFields { get; set; } = null;

        /// <summary>
        /// 是否自动生成查询条件
        /// </summary>
        public bool IsAutoGenerateCondition { get; set; } = false;

        /// <summary>
        /// 构造函数
        /// </summary>
        public PageRequest()
        {
            this.PageIndex = 1;
            this.PageSize = 10;
            this.IsAutoGenerateCondition = false;
        }
    }
}
