﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Reflection.Emit;
using System.Text;

namespace Wms.Screen.Dto.ZhongTian.Response
{
    /// <summary>
    /// 库存数据
    /// </summary>
    public class StockDto
    {
        public string ContainerCode { get; set; }
        public string WarehouseCode { get; set; }
        public string WarehouseName { get; set; }
        public string LocationCode { get; set; }
        public string MaterialCode { get; set; }
        public string MaterialName { get; set; }
        public string Describe { get; set; }
        public string BarCode { get; set; }
        public decimal Quantity { get; set; }
        /// <summary>
        /// 生产编号
        /// </summary>
        public string ProductCode { get; set; }
        /// <summary>
        /// 单据号
        /// </summary>
        public string WarehouseDocumentCode { get; set; }

        /// <summary>
        /// wms任务号
        /// </summary>
        public int TaskCode { get; set; }
    }
    /// <summary>
    /// 装箱信息返回参数
    /// </summary>
    public class ProductPackRule
    {
        /// <summary>
        /// 箱号
        /// </summary>
        public string BoxNo { get; set; }
        /// <summary>
        /// 装箱规则
        /// </summary>
        public string PackRule { get; set; }
        /// <summary>
        /// 装箱状态
        /// </summary>
        public string EnchaseState { get; set; }
        /// <summary>
        /// 工字轮型号
        /// </summary>
        public string HWSpec { get; set; }
        /// <summary>
        /// 满轮数量
        /// </summary>
        public int FullCount { get; set; }
        /// <summary>
        /// 空轮数量
        /// </summary>
        public int EmptyCount { get; set; }
        /// <summary>
        /// 是否允许焊点盘
        /// </summary>
        public string IsSolder { get; set; }
    }
    public class PTStockDto
    {
        public string BoilerNo { get; set; }
        public string MatCode { get; set; }
        public int Qty { get; set; }
        public decimal TolWQty { get; set; }
    }

    public class SXStockDto
    {
        public string ContGrpBarCode { get; set; }
        public int Tunnel { get; set; }
        public string Code { get; set; }
        public string BoxBarCode { get; set; }
        public string SkuCode { get; set; }
        public string MatCode { get; set; }
        public string Grade { get; set; }
        public string HWTypeCode { get; set; }

        /// <summary> 
        /// ContGrpType
        /// </summary> 
        public int? ContGrpType { get; set; }
        public string ContGrpTypeName
        {
            get
            {
                return  GetDescription((ContGrpType)ContGrpType);
            }
        }

        /// <summary> 
        /// InvStateCode
        /// </summary> 
        public string InvStateCode { get; set; }
        public string InvStateName
        {
            get
            {
                if (Enum.TryParse(typeof(SXInvState), InvStateCode, out var result))
                {
                    return GetDescription(((SXInvState)result));
                }
                return "";

            }
        }

        public string GetDescription(Enum  obj)
        {
            var fi = obj.GetType().GetField(obj.ToString());
            var arrDesc = (DescriptionAttribute[])fi.GetCustomAttributes(typeof(DescriptionAttribute), false);
            return arrDesc[0]?.Description;
        }
    }

    public class CurStockInfo
    {
        /// <summary>
        /// SKU
        /// </summary>
        public string SKU { get; set; }
        /// <summary>
        /// 
        /// </summary>
        public string MatCode { get; set; }
        /// <summary>
        /// 物料描述
        /// </summary>
        public string MatName { get; set; }
        /// <summary>
        /// 订单号
        /// </summary>
        public string DocNo { get; set; }
        /// <summary>
        /// 机器组
        /// </summary>
        public string WbGroupCode { get; set; }
        /// <summary>
        /// 绕向
        /// </summary>
        public string Wind { get; set; }
        /// <summary>
        /// 工字轮类型
        /// </summary>
        public string HWType { get; set; }
        /// <summary>
        /// 规则
        /// </summary>
        public string Rule { get; set; }
        /// <summary>
        /// 是否检测盘
        /// </summary>
        public string IsTorsChk { get; set; }
        /// <summary>
        /// 是否黑盘
        /// </summary>
        public string IsBlack { get; set; }

        /// <summary>
        /// 质量等级
        /// </summary>
        public string Grade { get; set; }
        /// <summary>
        /// 数量
        /// </summary>
        public int Count { get; set; }
        /// <summary>
        /// 绕向(R/L)
        /// </summary>
        public List<string> NoBCount { get; set; }
        /// <summary>
        /// 黑盘数量(R/L)
        /// </summary>
        public List<string> BCount { get; set; }
        /// <summary>
        /// 是否有黑盘
        /// </summary>
        public List<string> HaveBlack { get; set; }
        /// <summary>
        /// 已装箱数
        /// </summary>
        public int FinCount { get; set; }
        /// <summary>
        /// 未装箱数
        /// </summary>
        public int NoFinCount { get; set; }
        /// <summary>
        /// 工字轮条码
        /// </summary>
        public string ContGrpBarCode { get; set; }

        /// <summary>
        /// 今天(R/L)
        /// </summary>
        public List<string> TodayQty { get; set; }
        /// <summary>
        /// 昨天(R/L)
        /// </summary>
        public List<string> YesterdayQty { get; set; }
        /// <summary>
        /// 前天 (R/L)
        /// </summary>
        public List<string> BeforeYesterdayQty { get; set; }
        public DateTime ProductTime { get; set; }

    }
}
