﻿using AutoMapper;
using System.Collections.Generic;
using System.Linq;
using Wms.Screen.DataService.Interface;
using Wms.Screen.Dto.ZhongTian.Response.KLHC;
using Wms.Screen.Service.IService;
using Wms.Screen.SqlSugar.ZhongTian.KLHC;

namespace Wms.Screen.Service.Service
{
    /// <summary>
    /// 空轮缓存库service实现
    /// </summary>
    public class ZhongTianKlhcService : IZhongTianKlhcService
    {
        /// <summary>
        /// 空轮缓存库数据服务
        /// </summary>
        private IZhongTianKlhcDataService _zhongTianSxDataService;

        /// <summary>
        /// AutoMapper对象
        /// </summary>
        private IMapper _Mapper;

        /// <summary>
        /// Redis客户端对象
        /// </summary>
        private readonly FreeRedis.RedisClient _freeRedisClient;

        /// <summary>
        /// 构造函数
        /// </summary>
        /// <param name="zhongTianSxDataService">空轮缓存库数据服务</param>
        /// <param name="mapper">AutoMapper对象</param>
        /// <param name="freeRedisClient">Redis客户端对象</param>
        public ZhongTianKlhcService(IZhongTianKlhcDataService zhongTianSxDataService, IMapper mapper, FreeRedis.RedisClient freeRedisClient)
        {
            _zhongTianSxDataService = zhongTianSxDataService;
            _Mapper = mapper;
            _freeRedisClient = freeRedisClient;
        }

        /// <summary>
        /// 缓存信息
        /// </summary>
        /// <returns></returns>
        public List<BaseRBCI> GetBaseRrciList()
        {
            return _zhongTianSxDataService.GetBaseRrciList().Select(x =>
            {
                x.Direction = x.Direction == "N" ? "北" : "南";
                return x;
            }).OrderBy(x => x.Direction).ThenBy(x => int.Parse(x.WheelType)).ToList();
        }

        /// <summary>
        /// 库存信息
        /// </summary>
        /// <returns></returns>
        public List<KlhcInvInfoRes> GetKlhcInvInfo()
        {
            var infos = _zhongTianSxDataService.GetBaseWarecellList().GroupBy(x => x.WheelType).Select(x => new KlhcInvInfoRes { WheelType = x.Key, Number = x.Sum(s => s.Number) }).OrderBy(x => int.Parse(x.WheelType)).ToList();
            infos.Add(new KlhcInvInfoRes { WheelType = "总计", Number = infos.Sum(x => x.Number) });
            return infos;
        }
    }
}