﻿using System; 
using System.Collections.Generic;
using SqlSugar;
using Wms.Screen.SqlSugar;

namespace wms.sqlsugar.model.fj
{

    /// <summary>
    /// 任务表
    /// </summary>
    [Tenant("fj")]
    [SugarTable(nameof(WCS_TaskOld) + "_{year}{month}{day}", "任务表")]
    [SplitTable(SplitType.Month)]//按年分表 （自带分表支持 年、季、月、周、日）
    public class WCS_TaskOld : BaseEntityModel
    {
        /// <summary>
        /// ID
        /// </summary>
        [SugarColumn(IsPrimaryKey = true, ColumnDescription = "ID")]
        public int ID { get; set; }

        
        /// <summary> 
        /// 生产编码
        /// </summary> 
        [SugarColumn(ColumnDataType = "varchar", Length = 50, IsNullable = true, ColumnDescription = "生产编码")]
        public string BarCode { get; set; }

        /// <summary> 
        /// 任务类型
        /// </summary> 
        [SugarColumn(ColumnDataType = "int", IsNullable = true, ColumnDescription = "任务类型")]
        public int? Type { get; set; }

        /// <summary> 
        /// 任务状态
        /// </summary> 
        [SugarColumn(ColumnDataType = "int", IsNullable = true, ColumnDescription = "任务状态")]
        public int? Status { get; set; }

        /// <summary> 
        /// 起始地址
        /// </summary> 
        [SugarColumn(ColumnDataType = "varchar", Length = 20, IsNullable = true, ColumnDescription = "起始地址")]
        public string AddrFrom { get; set; }

        /// <summary> 
        /// 目标地址
        /// </summary> 
        [SugarColumn(ColumnDataType = "varchar", Length = 20, IsNullable = true, ColumnDescription = "目标地址")]
        public string AddrTo { get; set; }

        /// <summary> 
        /// 最后一个交互点
        /// </summary> 
        [SugarColumn(ColumnDataType = "varchar", Length = 20, IsNullable = true, ColumnDescription = "最后一个交互点")]
        public string LastInteractionPoint { get; set; }

        /// <summary> 
        /// 下一个地址
        /// </summary> 
        [SugarColumn(ColumnDataType = "varchar", Length = 20, IsNullable = true, ColumnDescription = "下一个地址")]
        public string AddrNext { get; set; }

        /// <summary> 
        /// 巷道
        /// </summary> 
        [SugarColumn(ColumnDataType = "varchar", Length = 10, IsNullable = true, ColumnDescription = "巷道")]
        public string Tunnel { get; set; }

        /// <summary> 
        /// 楼层
        /// </summary> 
        [SugarColumn(ColumnDataType = "int", IsNullable = false, ColumnDescription = "楼层")]
        public int Floor { get; set; }

        /// <summary> 
        /// 设备
        /// </summary> 
        [SugarColumn(ColumnDataType = "varchar", Length = 20, IsNullable = true, ColumnDescription = "设备")]
        public string Device { get; set; }

        /// <summary> 
        /// 堆垛机放货站台
        /// </summary> 
        [SugarColumn(ColumnDataType = "varchar", Length = 20, IsNullable = true, ColumnDescription = "堆垛机放货站台")]
        public string SrmStation { get; set; }

        /// <summary> 
        /// 创建时间
        /// </summary> 
        [SugarColumn(ColumnDataType = "datetime", IsNullable = false, ColumnDescription = "创建时间")]
        [SplitField]
        public DateTime AddTime { get; set; }

        /// <summary> 
        /// 开始时间
        /// </summary> 
        [SugarColumn(ColumnDataType = "datetime", IsNullable = true, ColumnDescription = "开始时间")]
        public DateTime? StartTime { get; set; }

        /// <summary> 
        /// 结束时间
        /// </summary> 
        [SugarColumn(ColumnDataType = "datetime", IsNullable = true, ColumnDescription = "结束时间")]
        public DateTime? EndTime { get; set; }

        /// <summary> 
        /// 客户现场的机台/产线编号
        /// </summary> 
        [SugarColumn(ColumnDataType = "varchar", Length = 255, IsNullable = true, ColumnDescription = "客户现场的机台/产线编号")]
        public string WorkBench { get; set; }

        /// <summary> 
        /// 单号
        /// </summary> 
        [SugarColumn(ColumnDataType = "int", IsNullable = true, ColumnDescription = "单号")]
        public int? DocID { get; set; }

        /// <summary> 
        /// 托盘类型
        /// </summary> 
        [SugarColumn(ColumnDataType = "smallint", IsNullable = true, ColumnDescription = "托盘类型")]
        public int? PalletType { get; set; }

        /// <summary> 
        /// 码垛工位
        /// </summary> 
        [SugarColumn(ColumnDataType = "smallint", IsNullable = true, ColumnDescription = "码垛工位")]
        public int? ProdLine { get; set; }

        /// <summary> 
        /// 满盘数量
        /// </summary> 
        [SugarColumn(ColumnDataType = "smallint", IsNullable = true, ColumnDescription = "满盘数量")]
        public int? FullQty { get; set; }

        /// <summary> 
        /// 高度
        /// </summary> 
        [SugarColumn(ColumnDataType = "smallint", IsNullable = true, ColumnDescription = "高度")]
        public int? Height { get; set; }

        /// <summary> 
        /// WMS任务号
        /// </summary> 
        [SugarColumn(ColumnDataType = "int", IsNullable = false, ColumnDescription = "WMS任务号")]
        public int WmsTask { get; set; }

        /// <summary> 
        /// 任务组ID
        /// </summary> 
        [SugarColumn(ColumnDataType = "varchar", Length = 255, IsNullable = true, ColumnDescription = "任务组ID")]
        public string TaskGroupKey { get; set; }

        /// <summary> 
        /// 更新状态
        /// </summary> 
        [SugarColumn(ColumnDataType = "int", IsNullable = false, ColumnDescription = "更新状态")]
        public int Uploaded { get; set; }

        /// <summary> 
        /// AGV任务号
        /// </summary> 
        [SugarColumn(ColumnDataType = "int", IsNullable = true, ColumnDescription = "AGV任务号")]
        public int? AgvTaskID { get; set; }

        /// <summary> 
        /// 优先级
        /// </summary> 
        [SugarColumn(ColumnDataType = "int", IsNullable = false, ColumnDescription = "优先级")]
        public int Priority { get; set; }

        /// <summary> 
        /// 手动处理备注
        /// </summary> 
        [SugarColumn(ColumnDataType = "varchar", Length = 255, IsNullable = true, ColumnDescription = "手动处理备注")]
        public string ManualRemarks { get; set; }

        /// <summary> 
        /// 产品长度
        /// </summary> 
        [SugarColumn(ColumnDataType = "decimal", Length = 18, IsNullable = true, ColumnDescription = "产品长度")]
        public decimal? Length { get; set; }

        /// <summary> 
        /// 产品编码
        /// </summary> 
        [SugarColumn(ColumnDataType = "varchar", Length = 255, IsNullable = true, ColumnDescription = "产品编码")]
        public string MaterialCode { get; set; }

        /// <summary> 
        /// 创建用户
        /// </summary> 
        [SugarColumn(ColumnDataType = "varchar", Length = 50, IsNullable = false, ColumnDescription = "创建用户")]
        public string AddWho { get; set; }

        /// <summary> 
        /// 更新用户
        /// </summary> 
        [SugarColumn(ColumnDataType = "varchar", Length = 50, IsNullable = true, ColumnDescription = "更新用户")]
        public string EditWho { get; set; }

        /// <summary> 
        /// 更新时间
        /// </summary> 
        [SugarColumn(ColumnDataType = "datetime", IsNullable = true, ColumnDescription = "更新时间")]
        public DateTime EditTime { get; set; }

        /// <summary> 
        /// 仓库编号
        /// </summary> 
        [SugarColumn(ColumnDataType = "varchar", Length = 255, IsNullable = true, ColumnDescription = "仓库编号")]
        public string WarehouseCode { get; set; }

        /// <summary> 
        /// 物料编码
        /// </summary> 
        [SugarColumn(ColumnDataType = "nvarchar", Length = 50, IsNullable = true, ColumnDescription = "物料编码")]
        public string MatCode { get; set; }

        /// <summary> 
        /// 材料号
        /// </summary> 
        [SugarColumn(ColumnDataType = "nvarchar", Length = 50, IsNullable = true, ColumnDescription = "材料号")]
        public string InvBarCode { get; set; }

        /// <summary> 
        /// 是否快投
        /// </summary> 
        [SugarColumn(ColumnDataType = "bit", IsNullable = true, ColumnDescription = "是否快投")]
        public bool? IsFast { get; set; }

        /// <summary> 
        /// 等级
        /// </summary> 
        [SugarColumn(ColumnDataType = "nvarchar", Length = 50, IsNullable = true, ColumnDescription = "等级")]
        public string Grade { get; set; }

        /// <summary> 
        /// 是否改手盘
        /// </summary> 
        [SugarColumn(ColumnDataType = "bit", IsNullable = true, ColumnDescription = "是否改手盘")]
        public bool? IsSurplus { get; set; }

        /// <summary> 
        /// 是否返空
        /// </summary> 
        [SugarColumn(ColumnDataType = "bit", IsNullable = true, ColumnDescription = "是否返空")]
        public bool? IsRework { get; set; }

        /// <summary> 
        /// 出库任务类型
        /// </summary> 
        [SugarColumn(ColumnDataType = "int", IsNullable = true, ColumnDescription = "出库任务类型")]
        public int? OutType { get; set; }

        /// <summary> 
        /// 重量
        /// </summary> 
        [SugarColumn(ColumnDataType = "float", IsNullable = true, ColumnDescription = "重量")]
        public double? Weight { get; set; }
        /// <summary>
        /// 业务类型
        /// </summary>
        [SugarColumn(ColumnDescription = "业务类型", IsNullable = true)]
        public string BusType { get; set; }

        /// <summary> 
        /// 生产单号
        /// </summary> 
        [SugarColumn(ColumnDataType = "varchar", Length = 255, IsNullable = true, ColumnDescription = "生产单号")]
        public string WorkOrder { get; set; }
        /// <summary> 
        ///货物大小
        /// </summary> 
        [SugarColumn(ColumnDataType = "int", IsNullable = true, ColumnDescription = "货位类型")]
        public int? GoodsType { get; set; }

        /// <summary> 
        /// 行
        /// </summary> 
        [SugarColumn(ColumnDataType = "int", IsNullable = true, ColumnDescription = "行")]
        public int? Line { get; set; }

        /// <summary> 
        /// 列
        /// </summary> 
        [SugarColumn(ColumnDataType = "int", IsNullable = true, ColumnDescription = "列")]
        public int? Col { get; set; }

        /// <summary> 
        /// 层
        /// </summary> 
        [SugarColumn(ColumnDataType = "int", IsNullable = true, ColumnDescription = "层")]
        public int? Layer { get; set; }

        /// <summary> 
        /// 深度
        /// </summary> 
        [SugarColumn(ColumnDataType = "int", IsNullable = true, ColumnDescription = "深度")]
        public int? Depth { get; set; }

    }
}

