﻿using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Data;
using System.Reflection;
using Wms.Screen.Dto;

namespace Wms.Screen.Util.Extensions
{
    public static class ResultExt
    {
        public static string ToCamelCaseString(this Result result)
        {
            return JsonConvert.SerializeObject(result, Formatting.Indented, new JsonSerializerSettings
            {
                ContractResolver = new Newtonsoft.Json.Serialization.CamelCasePropertyNamesContractResolver()
            });
        }
    }

    public static class Dtbl2ListExt
    {
        public static List<T> ToList<T>(DataTable dt)
        {
            List<T> list = new List<T>();
            Type t = typeof(T);
            List<PropertyInfo> plist = new List<PropertyInfo>(typeof(T).GetProperties());
            if (dt != null)
            {
                foreach (DataRow item in dt.Rows)
                {//如果是单行转对应Model只要这一部分
                    T s = System.Activator.CreateInstance<T>();
                    for (int i = 0; i < dt.Columns.Count; i++)
                    {
                        PropertyInfo info = plist.Find(p => p.Name == dt.Columns[i].ColumnName);
                        if (info != null)
                        {
                            if (!Convert.IsDBNull(item[i]))
                            {
                                info.SetValue(s, item[i], null);
                            }
                        }
                    }
                    list.Add(s);
                }
            }
            return list;
        }


        public static DataTable PageDataTable(DataTable dt, int PageIndex, int PageSize)
        {
            DataTable newdt = dt.Copy();
            newdt.Clear();//copy dt的框架

            int rowbegin = (PageIndex - 1) * PageSize;
            int rowend = PageIndex * PageSize;//要展示的数据条数

            if (rowbegin >= dt.Rows.Count)
                return newdt;//源数据记录数小于等于要显示的记录，直接返回dt

            if (rowend > dt.Rows.Count)
                rowend = dt.Rows.Count;
            for (int i = rowbegin; i <= rowend - 1; i++)
            {
                DataRow newdr = newdt.NewRow();
                DataRow dr = dt.Rows[i];
                foreach (DataColumn column in dt.Columns)
                {
                    newdr[column.ColumnName] = dr[column.ColumnName];
                }
                newdt.Rows.Add(newdr);
            }
            return newdt;
        }

    }

    //public static class WcsApiResponseExt
    //{
    //    public static string ToPhyString(this WcsContractApiResponse result)
    //    {
    //        return JsonConvert.SerializeObject(result, Formatting.Indented, new JsonSerializerSettings
    //        {
    //            ContractResolver = new Newtonsoft.Json.Serialization.DefaultContractResolver()
    //        });
    //    }
    //}
    //public static class ListExt
    //{
    //    public static string ToPhyString(this List<I_WCS_PutTaskStepResponse> result)
    //    {
    //        return JsonConvert.SerializeObject(result, Formatting.Indented, new JsonSerializerSettings
    //        {
    //            ContractResolver = new Newtonsoft.Json.Serialization.DefaultContractResolver()
    //        });
    //    }
    //}
}
