﻿using AutoMapper;
using Bozhon.Wms.Util.Cache;
using Microsoft.AspNetCore.Mvc;
using Newtonsoft.Json;
using System.Collections.Generic;
using System;
using Wms.Screen.Dto.ZhongTian.Request;
using Wms.Screen.Dto.ZhongTian.Response;
using Wms.Screen.Dto.ZhongTian;
using Wms.Screen.Service.IService;
using System.Linq;
using Wms.Screen.Dto;
using Wms.Screen.Service.Service;
using CSRedis;
using zt.screen.api.Extensions;
using Microsoft.Extensions.Options;

namespace zt.screen.api.Controllers
{
    /// <summary>
    /// 中天成品看板
    /// </summary>
    [Route("api/[controller]/[action]")]
    [ApiController]
    public class ZhongTianCpController : ControllerBase
    {
        private readonly MySettings _mySettings;
        private IZhongTianCpService _zhongTianCpService;
        private IMapper _mapper;
        private readonly CSRedisClient _CSRedisClient;
        public ZhongTianCpController(IZhongTianCpService zhongTianCpService, IMapper mapper, CSRedisClient cSRedisClient, IOptions<MySettings> mySettings)
        {
            _zhongTianCpService = zhongTianCpService;
            _mapper = mapper;
            _CSRedisClient = cSRedisClient;
            _mySettings = mySettings.Value;
        }

        #region 查询配置文件信息
        /// <summary>
        /// 查询配置的车间设备信息
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public List<WorkShopWithEquipment> GetWorkShopWithEquipment()
        {
            List<WorkShopWithEquipment> workShopWithEquipmentList = new List<WorkShopWithEquipment>();
            workShopWithEquipmentList.AddRange(InOutEquipment.WorkShopWithEquipment);
            workShopWithEquipmentList.AddRange(InOutEquipment.YTWorkShopWithEquipment);
            return workShopWithEquipmentList;
        }
        #endregion

        #region 成品看板
        /// <summary>
        /// 成品看板
        /// </summary>
        /// <param name="strRequest"></param>
        /// <param name="workshop"></param>
        /// <param name="boardCode"></param>
        /// <returns></returns>
        [HttpPost]
        public List<dynamic> GetCpInfo(StrRequestDto request)
        {
            if (string.IsNullOrEmpty(request.StrRequest))
            {
                return new List<dynamic> { new object(), new object(), new object(), new object(), new object() };
            }
            var result = new List<dynamic>();

            if (request.StrRequest.Split(',').Contains("1"))  //查询设备状态
            {
                result.Add(_zhongTianCpService.GetCpEquips(new GetEquipsRequest()
                {

                }, _CSRedisClient));
            }
            else
            {
                result.Add("");
            }
            if (request.StrRequest.Split(',').Contains("2")) //库存统计占比
            {
                result.Add(_zhongTianCpService.GetTrayProductList());
            }
            else
            {
                result.Add("");
            }
            if (request.StrRequest.Split(',').Contains("3")) //任务信息
            {
                var taskInfos = _zhongTianCpService.GetCpTaskInfo(new GetWcsTaskInfoRequest()
                {
                    TaskTypelist = new List<TaskType> { TaskType.OutDepot, TaskType.EnterDepot }
                });
                result.Add(taskInfos);
            }
            else
            {
                result.Add("");
            }
            if (request.StrRequest.Split(',').Contains("4")) //报警信息
            {
                List<string> equipNoList = new List<string>();
                InOutEquipment.WorkShopWithEquipment.Where(s => s.InOrOut == "Out" && InOutEquipment.DeopWithWOrkShops.Where(m => m.Dept == request.Workshop).FirstOrDefault().WorkShopList.Contains(s.WorkShop)).ToList().ForEach(
                    n => equipNoList.AddRange(n.Equipments));
                //获取报警信息
                //设备号数据，做成配置
                result.Add(GetStockInAndOutRecordInfo(new GetEquipsRequest
                {
                    EquCodeList = equipNoList.Distinct().ToList()
                }));
            }
            else
            {
                result.Add("");
            }
            if (request.StrRequest.Split(',').Contains("6")) //库存统计占比
            {
                result.Add(_zhongTianCpService.GetWarningQtyList(_mySettings));
            }
            else
            {
                result.Add("");
            }
            return result;
        }
        #endregion

        #region 货位利用率
        /// <summary>
        /// 成品货位利用率
        /// </summary>
        /// <param name="strRequest"></param>
        /// <returns></returns>
        [HttpPost]
        public PageResult<ZtLocationUsageReportViewDto> GetCpLocationStateList()
        {
            return _zhongTianCpService.GetCpLocationUsageViewList();
        }
        #endregion

        /// <summary>
        ///  获取产品信息
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public DeliveryDetails GetProductInfoList()
        {
            var value=_zhongTianCpService.GetProductInfoList();
            return value;
        }

        #region 异常信息汇总
        /// <summary>
        /// 成品异常信息汇总
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public List<dynamic> GetTotalErrorInfo()
        {
            var result = new List<dynamic>();
            List<string> equipNoList = new List<string>();
            InOutEquipment.WorkShopWithEquipment.Where(s => InOutEquipment.DeopWithWOrkShops.Where(m => m.Dept == "cp").FirstOrDefault().WorkShopList.Contains(s.WorkShop)).ToList().ForEach(
                n => equipNoList.AddRange(n.Equipments));
            //报警信息
            result.Add(GetStockInAndOutRecordInfo(new GetEquipsRequest()
            {
                EquCodeList = equipNoList.Distinct().ToList()
            }));
            return result;
        }
        #endregion

        #region 私有方法：将报表方法提取出来
        /// <summary>
        /// 报警信息
        /// </summary>
        /// <param name="reqEntity"></param>
        /// <returns></returns>
        private List<StockInAndOutRecordDto> GetStockInAndOutRecordInfo(GetEquipsRequest reqEntity)
        {
            var list = new List<StockInAndOutRecordDto>();
            try
            {
                foreach (var equipno in reqEntity.EquCodeList)
                {
                    var msg1 = RedisHelper.Get("Cp:EquipmentAlarm:" + equipno);  //设备报警

                    if (!string.IsNullOrEmpty(msg1))
                    {
                        var msgdto = JsonConvert.DeserializeObject<I_WCS_PutDevInfoRequestDto>(msg1);
                        if (msgdto.Msg == "0" || msgdto.Msg == "无")
                        {
                            continue;
                        }
                        if (!string.IsNullOrEmpty(msgdto.Msg) && msgdto.Time.Date == DateTime.Now.Date)
                            list.Add(new StockInAndOutRecordDto { dateTime = msgdto.Time.ToString("HH:mm:ss"), Message = msgdto.Code + "，" + msgdto.Msg });
                    }
                }
                var msg2 = _CSRedisClient.Get("Cp:BusinessAlarm:MainWorld");  //业务报警
                var msgdto2 = JsonConvert.DeserializeObject<List<BusinessDto>>(msg2);
                if (msgdto2.Any())
                {
                    foreach (var item in msgdto2)
                    {
                        if (!string.IsNullOrEmpty(item.Con) && item.Time.Date == DateTime.Now.Date)
                        {
                            item.Con = item.Con.Substring(item.Con.LastIndexOf("内容:") + 3);
                            if (item.Con.StartsWith("[SRM1]") || item.Con.StartsWith("[SRM2]") || item.Con.StartsWith("[SRM3]") || item.Con.StartsWith("[SRM4]") || item.Con.StartsWith("[SRM5]"))
                            {
                                item.Con = item.Con.Substring(item.Con.LastIndexOf("]") + 1);
                            }
                            else if (item.Con.StartsWith("1011") || item.Con.StartsWith("1012") || item.Con.StartsWith("1014") || item.Con.StartsWith("1016"))
                            {
                                item.Con = item.Con.Substring(4);
                            }
                            else if (item.Con.StartsWith("SRM1") || item.Con.StartsWith("SRM2") || item.Con.StartsWith("SRM3") || item.Con.StartsWith("SRM4") || item.Con.StartsWith("SRM5"))
                            {
                                item.Con = item.Con.Substring(4);
                            }
                            if(reqEntity.EquCodeList.Contains(item.DevNo))
                            list.Add(new StockInAndOutRecordDto { dateTime = item.Time.ToString("HH:mm:ss"), Message = item.DevNo + "，" + item.Con });
                        }
                    }
                }

                //var msg3 = RedisHelper.Get("Alloy:BusinessAlarm:NoInteractionWorld");  //AGV交互报错信息
                //var msgdto3 = JsonConvert.DeserializeObject<List<BusinessDto>>(msg3);
                //if (msgdto3.Any())
                //{
                //    List<StockInAndOutRecordDto> agv = new List<StockInAndOutRecordDto>();
                //    foreach (var item in msgdto3)
                //    {
                //        if (!string.IsNullOrEmpty(item.Con) && (DateTime.Now - item.Time).Minutes.ToString().GetHashCode() < 2)
                //        {
                //            var mes = item.Con.Substring(item.Con.LastIndexOf("内容:") + 3);
                //            if (mes.StartsWith("[1011]--"))
                //            {
                //                mes = mes.Substring(mes.LastIndexOf("--") + 2);
                //            }
                //            agv.Add(new StockInAndOutRecordDto { dateTime = item.Time.ToString("HH:mm:ss"), Message = "海康报错信息" + "，" + mes });
                //        }
                //    }
                //    if (agv.Any())
                //    {
                //        var agv1 = agv.Select(p => p.Message).Distinct().ToList();
                //        foreach (var item in agv1)
                //        {
                //            list.Add(new StockInAndOutRecordDto { dateTime = DateTime.Now.ToString("HH:mm:ss"), Message = item });
                //        }
                //    }
                //}

            }
            catch (Exception ex)
            {
                list.Add(new StockInAndOutRecordDto { dateTime = DateTime.Now.ToString("HH:mm:ss"), Message = $"缓存报警读取失败:{ex.Message}" });
            }

            if (list.Any())
            {
                list = list.OrderByDescending(p => p.Code).ToList();
            }
            return list;
        }
        #endregion
    }

    
}
