﻿using AutoMapper;
using Bozhon.Wms.Util.Cache;
using Microsoft.AspNetCore.Mvc;
using Newtonsoft.Json;
using System.Collections.Generic;
using System;
using Wms.Screen.Dto.ZhongTian.Request;
using Wms.Screen.Dto.ZhongTian.Response;
using Wms.Screen.Dto.ZhongTian;
using Wms.Screen.Service.IService;
using System.Linq;
using Wms.Screen.Dto;
using Wms.Screen.DataService.Impl;

namespace zt.screen.api.Controllers
{
    /// <summary>
    /// 中天分拣看板
    /// </summary>
    [Route("api/[controller]/[action]")]
    [ApiController]
    public class ZhongTianFjController : ControllerBase
    {
        private IZhongTianFjService _zhongTianFjService;
        private IMapper _mapper;
        private Dictionary<string, RangeChecker> _dicRangeChecker;
        Dictionary<string, List<string>> _dicDevices;

        private readonly FreeRedis.RedisClient _freeRedisClient;
        public ZhongTianFjController(IZhongTianFjService zhongTianFjService, IMapper mapper, Dictionary<string, RangeChecker> dicRangeChecker, Dictionary<string, List<string>> dicDevices, FreeRedis.RedisClient freeRedisClient)
        {
            _zhongTianFjService = zhongTianFjService;
            _mapper = mapper;
            _dicRangeChecker = dicRangeChecker;
            _dicDevices = dicDevices;
            _freeRedisClient = freeRedisClient;
        }

        #region 查询配置文件信息
        /// <summary>
        /// 查询配置的车间设备信息
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public List<WorkShopWithEquipment> GetWorkShopWithEquipment()
        {
            List<WorkShopWithEquipment> workShopWithEquipmentList = new List<WorkShopWithEquipment>();
            workShopWithEquipmentList.AddRange(InOutEquipment.WorkShopWithEquipment);
            workShopWithEquipmentList.AddRange(InOutEquipment.YTWorkShopWithEquipment);
            return workShopWithEquipmentList;
        }
        #endregion

        #region 分拣看板
        /// <summary>
        /// 分拣看板
        /// </summary>
        /// <param name="strRequest"></param>
        /// <param name="workshop"></param>
        /// <param name="boardCode"></param>
        /// <returns></returns>
        [HttpPost]
        public List<dynamic> GetFjInfo(StrRequestDto request)
        {
            if (string.IsNullOrEmpty(request.StrRequest))
            {
                return new List<dynamic> { new object(), new object(), new object(), new object(), new object() };
            }
            var result = new List<dynamic>();
            if (request.StrRequest.Split(',').Contains("1"))  //堆垛机状态
            {
                //查询设备状态
                result.Add(_zhongTianFjService.GetFjEquips(new GetEquipsRequest()
                {
                    WarehouseCodeList = new List<string>() {
                     Const.Fjhouse_putong,request.Workshop}
                }));
            }
            else
            {
                result.Add("");
            }
            if (request.StrRequest.Split(',').Contains("2")) //码垛信息
            {
                result.Add(_zhongTianFjService.GetMantuoBarCodeRate(request.Workshop));

            }
            else
            {
                result.Add("");
            }
            if (request.StrRequest.Split(',').Contains("3")) //任务信息
            {
                result.Add(_zhongTianFjService.GetBomSetGrpCurrentMaterialRateList(request.Workshop));
            }
            else
            {
                result.Add("");
            }
            if (request.StrRequest.Split(',').Contains("4")) //报警信息
            {
                List<string> equipNoList = new List<string>();
                InOutEquipment.WorkShopWithEquipment.Where(s => s.InOrOut == "Out" && InOutEquipment.DeopWithWOrkShops.FirstOrDefault().WorkShopList.Contains(s.WorkShop)).ToList().ForEach(
                    n => equipNoList.AddRange(n.Equipments));
                //获取报警信息
                //设备号数据，做成配置
                result.Add(GetStockInAndOutRecordInfo(new GetEquipsRequest
                {
                    EquCodeList = equipNoList.Distinct().ToList()
                }, request.Workshop));

            }
            else
            {
                result.Add("");
            }

            return result;
        }


        /// <summary>
        /// 分拣看板（统计信息）
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public List<dynamic> GetFjStatsInfo(StrRequestDto request)
        {
            if (string.IsNullOrEmpty(request.StrRequest))
            {
                return new List<dynamic> { new object(), new object(), new object(), new object(), new object() };
            }
            var result = new List<dynamic>();
            if (request.StrRequest.Split(',').Contains("1"))  //货位分布占比
            {

                result.Add(_zhongTianFjService.GetLocaitonList(request.Workshop));
            }
            else
            {
                result.Add("");
            }
            if (request.StrRequest.Split(',').Contains("2")) //空满轮占比
            {
                //领料 涂布领料汇总，不区分车间
                // result.Add(_zhongTianHjService.GetEmptyFullDist());
                result.Add("");
            }
            else
            {
                result.Add("");
            }
            if (request.StrRequest.Split(',').Contains("3")) //7天出入库统计
            {
                var taskInfos = _zhongTianFjService.GeHistoryTaskInfo(new GeFJHistoryTaskInfoRequest()
                {
                    WareHouse = request.Workshop,
                    StartTime = DateTime.Now.AddDays(-6),
                    EndTime = DateTime.Now
                });
                result.Add(taskInfos);

            }
            else
            {
                result.Add("");
            }
            if (request.StrRequest.Split(',').Contains("4")) //物料重量分布
            {
                //var taskInfos = _zhongTianHjService.GetFullWeightDist();
                //result.Add(taskInfos);
                var lists = _zhongTianFjService.GetInvSetGrpList();
                result.Add(lists);
            }
            else
            {
                result.Add("");
            }
            if (request.StrRequest.Split(',').Contains("5")) //报警信息
            {
                List<string> equipNoList = new List<string>();

                //获取报警信息
                //设备号数据，做成配置
                result.Add(GetStockInAndOutRecordInfo(new GetEquipsRequest
                {
                    EquCodeList = equipNoList.Distinct().ToList()
                }, request.Workshop));
            }
            else
            {
                result.Add("");
            }

            return result;
        }
        #endregion

        #region 货位利用率
        /// <summary>
        /// 分拣货位利用率
        /// </summary>
        /// <param name="strRequest"></param>
        /// <returns></returns>
        [HttpPost]
        public PageResult<ZtLocationUsageReportViewDto> GetFjLocationStateList(StrRequestDto request)
        {
            return _zhongTianFjService.GetFjLocationUsageViewList(request.Workshop);
        }
        #endregion

        #region 异常信息汇总
        /// <summary>
        /// 分拣异常信息汇总
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public List<dynamic> GetTotalErrorInfo(StrRequestDto request)
        {
            var result = new List<dynamic>();
            List<string> equipNoList = new List<string>();
            InOutEquipment.WorkShopWithEquipment.Where(s => InOutEquipment.DeopWithWOrkShops.Where(m => m.Dept == "fj1").FirstOrDefault().WorkShopList.Contains(s.WorkShop)).ToList().ForEach(
                n => equipNoList.AddRange(n.Equipments));
            //报警信息
            result.Add(GetStockInAndOutRecordInfo(new GetEquipsRequest()
            {
                EquCodeList = equipNoList.Distinct().ToList()
            }, request.Workshop));
            return result;
        }
        #endregion

        #region 私有方法：将报表方法提取出来
        /// <summary>
        /// 报警信息
        /// </summary>
        /// <param name="reqEntity"></param>
        /// <returns></returns>
        private List<StockInAndOutRecordDto> GetStockInAndOutRecordInfo(GetEquipsRequest reqEntity, string warehouse)
        {
            var list = new List<StockInAndOutRecordDto>();
            try
            {
                _dicRangeChecker.TryGetValue(warehouse, out var range);
                _dicDevices.TryGetValue(warehouse, out var lists);
                var keylists = RedisHelper.Keys("BusinessAlarm**");
                foreach (var key in keylists)
                {


                    var value = RedisHelper.Get(key);
                    var dtolist = JsonConvert.DeserializeObject<List<BusinessDto>>(value);
                    if (dtolist != null && dtolist.Any())
                    {
                        foreach (var item in dtolist)
                        {
                            if (int.TryParse(item.DevNo, out var devno))
                            {
                                if (range != null && range.IsInRanges(devno) && !string.IsNullOrEmpty(item.Con) && item.Time.Date == DateTime.Now.Date)
                                {
                                    item.Con = item.Con.Substring(item.Con.LastIndexOf("内容:") + 3);
                                    list.Add(new StockInAndOutRecordDto { dateTime = item.Time.ToString("HH:mm:ss"), Message = item.DevNo + "，" + item.Con });
                                }
                            }
                            else
                            {
                                if (lists != null && lists.Contains(item.DevNo) && !string.IsNullOrEmpty(item.Con) && item.Time.Date == DateTime.Now.Date)
                                {
                                    item.Con = item.Con.Substring(item.Con.LastIndexOf("内容:") + 3);
                                    list.Add(new StockInAndOutRecordDto { dateTime = item.Time.ToString("HH:mm:ss"), Message = item.DevNo + "，" + item.Con });
                                }
                                else if ( !string.IsNullOrEmpty(item.Con) && item.Time.Date == DateTime.Now.Date)
                                {                                     
                                    item.Con = item.Con.Substring(item.Con.LastIndexOf("内容:") + 3);
                                    list.Add(new StockInAndOutRecordDto { dateTime = item.Time.ToString("HH:mm:ss"), Message = item.DevNo + "，" + item.Con });
                                }
                            }
                        }
                    }
                }
                var alarm = _freeRedisClient.Get("Sorting:EquipmentAlarm");
                if (!string.IsNullOrWhiteSpace(alarm) && alarm != "[]")
                {
                    var dtolist = JsonConvert.DeserializeObject<List<BusinessDto3>>(alarm);
                    if (dtolist != null && dtolist.Any())
                    {
                        foreach (var msgdto in dtolist)
                        {
                            if (int.TryParse(msgdto.Code, out var devno))
                            {
                                if (range != null && range.IsInRanges(devno) && !string.IsNullOrEmpty(msgdto.Msg) && msgdto.Time.Date == DateTime.Now.Date)
                                {

                                    list.Add(new StockInAndOutRecordDto { dateTime = msgdto.Time.ToString("HH:mm:ss"), Message = msgdto.Code + "，" + msgdto.Msg });
                                }
                            }
                            if (lists != null && lists.Contains(msgdto.Code) && !string.IsNullOrEmpty(msgdto.Msg) && msgdto.Time.Date == DateTime.Now.Date)
                            {
                                list.Add(new StockInAndOutRecordDto { dateTime = msgdto.Time.ToString("HH:mm:ss"), Message = msgdto.Code + "，" + msgdto.Msg });
                            }

                        }
                    }
                }
                list = list.OrderByDescending(o => o.dateTime).ToList();
                //foreach (var equipno in reqEntity.EquCodeList)
                //{
                //    var msg1 = RedisHelper.Get("Alloy:EquipmentAlarm:" + equipno);  //设备报警

                //    if (!string.IsNullOrEmpty(msg1))
                //    {
                //        var msgdto = JsonConvert.DeserializeObject<I_WCS_PutDevInfoRequestDto>(msg1);
                //        if (msgdto?.Msg == "0" || msgdto?.Msg == "无")
                //        {
                //            continue;
                //        }
                //        if (msgdto != null && !string.IsNullOrEmpty(msgdto.Msg) && msgdto.Time.Date == DateTime.Now.Date)
                //            list.Add(new StockInAndOutRecordDto { dateTime = msgdto.Time.ToString("HH:mm:ss"), Message = msgdto.Code + "，" + msgdto.Msg });
                //    }

                //}
                //var msg2 = RedisHelper.Get("BusinessAlarm:SortingMainLineWorld");  //业务报警
                //var msgdto2 = JsonConvert.DeserializeObject<List<BusinessDto>>(msg2);
                //if (msgdto2 != null && msgdto2.Any())
                //{
                //    foreach (var item in msgdto2)
                //    {
                //        if (!string.IsNullOrEmpty(item.Con) && item.Time.Date == DateTime.Now.Date)
                //        {
                //            item.Con = item.Con.Substring(item.Con.LastIndexOf("内容:") + 3);
                //            if (item.Con.StartsWith("[SRM1]") || item.Con.StartsWith("[SRM2]") || item.Con.StartsWith("[SRM3]"))
                //            {
                //                item.Con = item.Con.Substring(item.Con.LastIndexOf("]") + 1);
                //            }
                //            else if (item.Con.StartsWith("1011") || item.Con.StartsWith("1012") || item.Con.StartsWith("1014") || item.Con.StartsWith("1016"))
                //            {
                //                item.Con = item.Con.Substring(4);
                //            }
                //            else if (item.Con.StartsWith("SRM1") || item.Con.StartsWith("SRM2") || item.Con.StartsWith("SRM3"))
                //            {
                //                item.Con = item.Con.Substring(4);
                //            }
                //            list.Add(new StockInAndOutRecordDto { dateTime = item.Time.ToString("HH:mm:ss"), Message = item.DevNo + "，" + item.Con });
                //        }
                //    }
                //}

                //var msg3 = RedisHelper.Get("Alloy:BusinessAlarm:NoInteractionWorld");  //AGV交互报错信息
                //var msgdto3 = JsonConvert.DeserializeObject<List<BusinessDto>>(msg3);
                //if (msgdto3 != null && msgdto3.Any())
                //{
                //    List<StockInAndOutRecordDto> agv = new List<StockInAndOutRecordDto>();

                //    foreach (var item in msgdto3)
                //    {
                //        if (!string.IsNullOrEmpty(item.Con) && (DateTime.Now - item.Time).Minutes.ToString().GetHashCode() < 2)
                //        {
                //            var mes = item.Con.Substring(item.Con.LastIndexOf("内容:") + 3);
                //            if (mes.StartsWith("[1011]--"))
                //            {
                //                mes = mes.Substring(mes.LastIndexOf("--") + 2);
                //            }
                //            agv.Add(new StockInAndOutRecordDto { dateTime = item.Time.ToString("HH:mm:ss"), Message = "海康报错信息" + "，" + mes });
                //        }
                //    }
                //    if (agv.Any())
                //    {
                //        var agv1 = agv.Select(p => p.Message).Distinct().ToList();
                //        foreach (var item in agv1)
                //        {
                //            list.Add(new StockInAndOutRecordDto { dateTime = DateTime.Now.ToString("HH:mm:ss"), Message = item });
                //        }
                //    }
                //}
            }
            catch (Exception ex)
            {
                list.Add(new StockInAndOutRecordDto { dateTime = DateTime.Now.ToString("HH:mm:ss"), Message = $"缓存报警读取失败:{ex.Message}" });
            }


            if (list.Any())
            {
                list = list.OrderBy(p => p.Code).Distinct().ToList();
            }
            return list;
        }
        #endregion
    }
}
