﻿using AutoMapper;
using Bozhon.Wms.Util.Cache;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using Wms.Screen.Dto;
using Wms.Screen.Dto.ZhongTian;
using Wms.Screen.Dto.ZhongTian.Request;
using Wms.Screen.Dto.ZhongTian.Response;
using Wms.Screen.Service.IService;
namespace zt.screen.api.Controllers
{
    /// <summary>
    ///  中天合金看板
    /// </summary>
    [Route("api/[controller]/[action]")]
    [ApiController]
    public class ZhongTianHjController : ControllerBase
    {
        private IZhongTianHjService _zhongTianHjService;
        private IMapper _mapper;
        /// <summary>
        /// 构造函数注入
        /// </summary>
        /// <param name="zhongTianHjService"></param>
        /// <param name="mapper"></param>
        /// <param name="redisClient"></param>
        public ZhongTianHjController(IZhongTianHjService zhongTianHjService, IMapper mapper)
        {
            _zhongTianHjService = zhongTianHjService;
            _mapper = mapper;
          
        }

        #region 查询配置文件信息
        /// <summary>
        /// 查询配置的车间设备信息
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public List<WorkShopWithEquipment> GetWorkShopWithEquipment()
        {
            List<WorkShopWithEquipment> workShopWithEquipmentList = new List<WorkShopWithEquipment>();
            workShopWithEquipmentList.AddRange(InOutEquipment.WorkShopWithEquipment);
            workShopWithEquipmentList.AddRange(InOutEquipment.YTWorkShopWithEquipment);

            return workShopWithEquipmentList;
        }

        #endregion

        #region 合金看板
        /// <summary>
        /// 合金看板
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public List<dynamic> GetHjInfo(StrRequestDto request)
        {
            if (string.IsNullOrEmpty(request.StrRequest))
            {
                return new List<dynamic> { new object(), new object(), new object(), new object(), new object() };
            }
            var result = new List<dynamic>();
            if (request.StrRequest.Split(',').Contains("1"))  //堆垛机状态
            {
                //查询设备状态  ，读redis
                result.Add(_zhongTianHjService.GetHjEquips(new GetEquipsRequest()
                {

                }));
            }
            else
            {
                result.Add("");
            }
            if (request.StrRequest.Split(',').Contains("2")) //单据信息
            {
                //领料 涂布领料汇总，不区分车间
                result.Add(_zhongTianHjService.GetHjBillList(new GetWorkPlanBillListRequest()
                {
                    //WarehouseDocumentType = DocType.DocType_HJ_CopperProductStockIn.GetHashCode().ToString(),
                    WarehouseDocumentStateList = new List<string>() { DocState.DocState_Create.GetHashCode().ToString(), DocState.DocState_Execute.GetHashCode().ToString() }
                }));

            }
            else
            {
                result.Add("");
            }
            if (request.StrRequest.Split(',').Contains("3")) //任务信息
            {
                //List<string> equipNoList = new List<string>();
                //InOutEquipment.WorkShopWithEquipment.Where(s => s.InOrOut == "Out" && InOutEquipment.DeopWithWOrkShops.Where(m => m.Dept == request.Workshop).FirstOrDefault().WorkShopList.Contains(s.WorkShop)).Distinct().ToList().ForEach(
                //    n => equipNoList.AddRange(n.Equipments));
                //任务信息:出库和入库任务
                //设备号数据，做成配置
                var taskInfos = _zhongTianHjService.GetHjTaskInfo(new GetWcsTaskInfoRequest()
                {
                    //EquipList = equipNoList,
                    TaskTypelist = new List<TaskType> { TaskType.OutDepot, TaskType.EnterDepot, TaskType.EmptyInit }
                });
                result.Add(taskInfos);
            }
            else
            {
                result.Add("");
            }
            if (request.StrRequest.Split(',').Contains("4")) //报警信息
            {
                List<string> equipNoList = new List<string>();
                InOutEquipment.WorkShopWithEquipment.Where(s => s.InOrOut == "Out" && InOutEquipment.DeopWithWOrkShops.Where(m => m.Dept == request.Workshop).FirstOrDefault().WorkShopList.Contains(s.WorkShop)).ToList().ForEach(
                    n => equipNoList.AddRange(n.Equipments));
                //获取报警信息
                //设备号数据，做成配置
                result.Add(GetStockInAndOutRecordInfo(new GetEquipsRequest
                {
                    EquCodeList = equipNoList.Distinct().ToList()
                }));
            }
            else
            {
                result.Add("");
            }

            return result;
        }

        /// <summary>
        /// 合金看板（统计信息）
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public List<dynamic> GetHjStatsInfo(StrRequestDto request)
        {
            if (string.IsNullOrEmpty(request.StrRequest))
            {
                return new List<dynamic> { new object(), new object(), new object(), new object(), new object() };
            }
            var result = new List<dynamic>();
            if (request.StrRequest.Split(',').Contains("1"))  //货位分布占比
            {
                result.Add(_zhongTianHjService.GetLocaitonList());
            }
            else
            {
                result.Add("");
            }
            if (request.StrRequest.Split(',').Contains("2")) //空满轮占比
            {
                //领料 涂布领料汇总，不区分车间
                result.Add(_zhongTianHjService.GetEmptyFullDist());

            }
            else
            {
                result.Add("");
            }
            if (request.StrRequest.Split(',').Contains("3")) //7天出入库统计
            {
                var taskInfos = _zhongTianHjService.GeHistoryTaskInfo(new GeHistoryTaskInfoRequest()
                {
                    BusTypeList = new List<string>() { "镀铜一楼取满", "二楼湿拉自动叫料出库" },
                    StartTime = DateTime.Now.AddDays(-7),
                    EndTime = DateTime.Now.AddDays(1)
                });
                result.Add(taskInfos);
            }
            else
            {
                result.Add("");
            }
            if (request.StrRequest.Split(',').Contains("4")) //物料重量分布
            {
                var taskInfos = _zhongTianHjService.GetFullWeightDist();
                result.Add(taskInfos);
            }
            else
            {
                result.Add("");
            }
            if (request.StrRequest.Split(',').Contains("5")) //报警信息
            {
                List<string> equipNoList = new List<string>();
                InOutEquipment.WorkShopWithEquipment.Where(s => s.InOrOut == "Out" && InOutEquipment.DeopWithWOrkShops.Where(m => m.Dept == request.Workshop).FirstOrDefault().WorkShopList.Contains(s.WorkShop)).ToList().ForEach(
                    n => equipNoList.AddRange(n.Equipments));
                //获取报警信息
                //设备号数据，做成配置
                result.Add(GetStockInAndOutRecordInfo(new GetEquipsRequest
                {
                    EquCodeList = equipNoList.Distinct().ToList()
                }));
            }
            else
            {
                result.Add("");
            }
            if (request.StrRequest.Split(',').Contains("6")) //空满轮占比
            {
                //领料 涂布领料汇总，不区分车间
                result.Add(_zhongTianHjService.GetTunelCountList());

            }
            else
            {
                result.Add("");
            }
            return result;
        }
        #endregion

        #region 货位利用率
        /// <summary>
        /// 合金货位利用率
        /// </summary>
        /// <param name="strRequest"></param>
        /// <returns></returns>
        [HttpPost]
        public PageResult<ZtLocationUsageReportViewDto> GetHjLocationStateList()
        {
            return _zhongTianHjService.GetHjLocationUsageViewList();
        }

        #endregion

        #region 异常信息汇总
        /// <summary>
        /// 合金异常信息汇总
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public List<dynamic> GetTotalErrorInfo()
        {
            var result = new List<dynamic>();
            List<string> equipNoList = new List<string>();
            //InOutEquipment.WorkShopWithEquipment.Where(s => InOutEquipment.DeopWithWOrkShops.Where(m => m.Dept == "hj").FirstOrDefault().WorkShopList.Contains(s.WorkShop)).ToList().ForEach(
            //    n => equipNoList.AddRange(n.Equipments));
            //报警信息
            result.Add(GetStockInAndOutRecordInfo(new GetEquipsRequest()
            {
                EquCodeList = equipNoList.Distinct().ToList()
            }));
            return result;
        }
        #endregion


        #region 私有方法：将报表方法提取出来

        /// <summary>
        /// 报警信息
        /// </summary>
        /// <param name="reqEntity"></param>
        /// <returns></returns>
        private List<StockInAndOutRecordDto> GetStockInAndOutRecordInfo(GetEquipsRequest reqEntity)
        {
            var list = new List<StockInAndOutRecordDto>();
            try
            {

                foreach (var equipno in reqEntity.EquCodeList)
                {
                    var msg1 = RedisHelper.Get("Alloy:EquipmentAlarm:" + equipno);  //设备报警

                    if (!string.IsNullOrEmpty(msg1))
                    {
                        var msgdto = JsonConvert.DeserializeObject<I_WCS_PutDevInfoRequestDto>(msg1);
                        if (msgdto.Msg == "0" || msgdto.Msg == "无")
                        {
                            continue;
                        }
                        if (!string.IsNullOrEmpty(msgdto.Msg) && msgdto.Time.Date == DateTime.Now.Date)
                            list.Add(new StockInAndOutRecordDto { dateTime = msgdto.Time.ToString("HH:mm:ss"), Message = msgdto.Code + "，" + msgdto.Msg });
                    }
                    //if (!string.IsNullOrEmpty(msg2))
                    //{
                    //    var msgdto = JsonConvert.DeserializeObject<I_WCS_PutDevInfoRequestDto>(msg2);
                    //    if (!string.IsNullOrEmpty(msgdto.STA_ALARMSMSG) && msgdto.InOutTime.Date == DateTime.Now.Date)
                    //    {
                    //        list.Add(new StockInAndOutRecordDto { dateTime = msgdto.InOutTime.ToString("HH:mm:ss"), Message = msgdto.STA_ALARMSMSG });
                    //    }
                    //}
                }
                var msg2 = RedisHelper.Get("Alloy:BusinessAlarm:MainWorld");  //业务报警
                var msgdto2 = JsonConvert.DeserializeObject<List<BusinessDto>>(msg2);
                if (msgdto2.Any())
                {
                    foreach (var item in msgdto2)
                    {
                        if (!string.IsNullOrEmpty(item.Con) && item.Time.Date == DateTime.Now.Date)
                        {
                            item.Con = item.Con.Substring(item.Con.LastIndexOf("内容:") + 3);
                            if (item.Con.StartsWith("[SRM1]") || item.Con.StartsWith("[SRM2]") || item.Con.StartsWith("[SRM3]"))
                            {
                                item.Con = item.Con.Substring(item.Con.LastIndexOf("]") + 1);
                            }
                            else if (item.Con.StartsWith("1011") || item.Con.StartsWith("1012") || item.Con.StartsWith("1014") || item.Con.StartsWith("1016"))
                            {
                                item.Con = item.Con.Substring(4);
                            }
                            else if (item.Con.StartsWith("SRM1") || item.Con.StartsWith("SRM2") || item.Con.StartsWith("SRM3"))
                            {
                                item.Con = item.Con.Substring(4);
                            }
                            list.Add(new StockInAndOutRecordDto { dateTime = item.Time.ToString("HH:mm:ss"), Message = item.DevNo + "，" + item.Con });
                        }
                    }
                }

                var msg3 = RedisHelper.Get("Alloy:BusinessAlarm:NoInteractionWorld");  //AGV交互报错信息
                var msgdto3 = JsonConvert.DeserializeObject<List<BusinessDto>>(msg3);
                if (msgdto3.Any())
                {
                    List<StockInAndOutRecordDto> agv = new List<StockInAndOutRecordDto>();
                    //var agvError = msgdto3.Select(p => p.Con).Distinct().ToList();
                    //foreach (var item in agvError)
                    //{
                    //    if (!string.IsNullOrEmpty(item))
                    //    {
                    //        var mes = item.Substring(item.LastIndexOf("内容:") + 3);
                    //        if (mes.StartsWith("[1011]--"))
                    //        {
                    //            mes = mes.Substring(mes.LastIndexOf("--") + 2);
                    //        }
                    //        list.Add(new StockInAndOutRecordDto { dateTime = DateTime.Now.ToString("HH:mm:ss"), Message = "海康报错信息" + "，" + mes });
                    //    }
                    //}
                    foreach (var item in msgdto3)
                    {
                        if (!string.IsNullOrEmpty(item.Con) && (DateTime.Now - item.Time).Minutes.ToString().GetHashCode() < 2)
                        {
                            var mes = item.Con.Substring(item.Con.LastIndexOf("内容:") + 3);
                            if (mes.StartsWith("[1011]--"))
                            {
                                mes = mes.Substring(mes.LastIndexOf("--") + 2);
                            }
                            agv.Add(new StockInAndOutRecordDto { dateTime = item.Time.ToString("HH:mm:ss"), Message = "海康报错信息" + "，" + mes });
                        }
                    }
                    if (agv.Any())
                    {
                        var agv1 = agv.Select(p => p.Message).Distinct().ToList();
                        foreach (var item in agv1)
                        {
                            list.Add(new StockInAndOutRecordDto { dateTime = DateTime.Now.ToString("HH:mm:ss"), Message = item });
                        }
                    }
                } 
            }
            catch (Exception ex)
            {
                list.Add(new StockInAndOutRecordDto { dateTime = DateTime.Now.ToString("HH:mm:ss"), Message = $"缓存报警读取失败:{ex.Message}" });
            }
            //for (int i = 0; i < list.Count; i++)
            //{
            //    var code = list[i].Message.Split('，');  //取设备号
            //    if (list.Where(p => p.Message.StartsWith(code[0])).Count() > 1)
            //    {
            //        var pro = list.Where(p => p.Message.StartsWith(code[0])).ToList();
            //        var pro1 = pro.Where(p => !p.Message.Contains(code[1])).ToList(); //取到同设备不同的报警信息
            //        for (int j = 0; j < pro.Count; j++)
            //        {
            //            if (pro[j].Message == list[i].Message)
            //            {
            //                continue;
            //            }
            //            var aa = pro[j].Message.Split('，');
            //            if (aa.Count() > 2)
            //            {
            //                if (list[i].Message.Contains(aa[2]))
            //                {
            //                    continue;
            //                }
            //            }
            //            var mes = pro[j].Message.Split('，');
            //            list[i].Message += "，" + mes[1];
            //        }
            //    }

            //}

            if (list.Any())
            {
                list = list.OrderBy(p => p.Code).Distinct().ToList();
            }
            return list;
        }
        #endregion
    }
}
