﻿using AutoMapper;
using Bozhon.Wms.Util.Cache;
using Microsoft.AspNetCore.Mvc;
using Newtonsoft.Json;
using System.Collections.Generic;
using System;
using Wms.Screen.Dto.ZhongTian.Request;
using Wms.Screen.Dto.ZhongTian.Response;
using Wms.Screen.Dto.ZhongTian;
using Wms.Screen.Service.IService;
using System.Linq;
using Wms.Screen.Dto;
using Wms.Screen.Service.Service;
using System.Security.Cryptography.X509Certificates;
using CSRedis;
using MessagePack.Resolvers;
using MessagePack;
using WCS.Entity.Protocol.DataStructure;
using System.Collections;
using Wms.Screen.DataService.Impl;
using WCS.Protocol.SX.DataStructure;
using WCS.Protocol.SX.QT;
namespace zt.screen.api.Controllers
{
    /// <summary>
    /// 中天时效看板
    /// </summary>
    [Route("api/[controller]/[action]")]
    [ApiController]
    public class ZhongTianSxController : ControllerBase
    {
        private IZhongTianSxService _zhongTianSxService;
        private IMapper _mapper;
        private readonly CSRedisClient _CSRedisClient;
        private readonly FreeRedis.RedisClient _freeRedisClient;
        public ZhongTianSxController(IZhongTianSxService zhongTianSxService, IMapper mapper, CSRedisClient cSRedisClient, FreeRedis.RedisClient freeRedisClient)
        {
            _zhongTianSxService = zhongTianSxService;
            _mapper = mapper;
            _CSRedisClient = cSRedisClient;

            _freeRedisClient = freeRedisClient;
        }

        #region 查询配置文件信息
        /// <summary>
        /// 查询配置的车间设备信息
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public List<WorkShopWithEquipment> GetWorkShopWithEquipment()
        {
            List<WorkShopWithEquipment> workShopWithEquipmentList = new List<WorkShopWithEquipment>();
            workShopWithEquipmentList.AddRange(InOutEquipment.WorkShopWithEquipment);
            workShopWithEquipmentList.AddRange(InOutEquipment.YTWorkShopWithEquipment);
            return workShopWithEquipmentList;
        }
        #endregion

        #region 扭转检测报警看板
        /// <summary>
        /// 获取扭转检测报警信息
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public PageResult<ReverseErrorInfo> GetReverseErrorInfo()
        {
            return _zhongTianSxService.GetReverseErrorInfo();
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [HttpPost]
        public List<dynamic> GetTorsiveTestInfo(StrRequestDto request)
        {
            if (string.IsNullOrEmpty(request.StrRequest)) 
            {
                return new List<dynamic> { new object(), new object(), new object() };
            }
            var result = new List<dynamic>();

            if (request.StrRequest.Split(',').Contains("1"))  //设备状态
            {
                //查询设备状态
                result.Add(_zhongTianSxService.GetTorsiveTestEquips(new GetEquipsRequest()
                {
                    WarehouseCodeList = new List<string>() {
                     Const.Sxhouse_putong }
                }));
            }
            else
            {
                result.Add("");
            }
            if (request.StrRequest.Split(',').Contains("2")) //统计信息
            {
                result.Add(_zhongTianSxService.GetTorStaticsInfo());
            }
            else
            {
                result.Add("");
            }
            if (request.StrRequest.Split(',').Contains("3")) //报警信息
            {
                result.Add(GetTorsiveTestStockInAndOutRecordInfo( ));
            }
            else
            {
                result.Add("");
            }

            return result;
        }

        private List<StockInAndOutRecordDto> GetTorsiveTestStockInAndOutRecordInfo( )
        {
            var list = new List<StockInAndOutRecordDto>();
            
            try
            {
                var Len = _freeRedisClient.LLen("Packs");
                var dataPack = _freeRedisClient.LIndex<DataPack>("Packs", Len - 1);
                var qtlists = dataPack.QT51.Where(qt => qt.Frame.Date == DateTime.Now.Date &&  
                !(qt.Data.Enable == QTStatus.启用 && qt.Data.RunStatus == QTRunStatus.空闲 && qt.Data.Fault == QTFault.非异常)
                || !(qt.Data.RunStatus == QTRunStatus.非空闲 || qt.Data.Enable == QTStatus.停用) )
                    .Select(o => new StockInAndOutRecordDto { dateTime = o.Frame.ToString("HH:mm:ss"), Message = o.Code + "，" + o.Data.Fault.ToString() }).ToList();
               
                list.AddRange(qtlists);

                var first = new RangeChecker();
                first.AddRange(new IntRange(4000, 4275));
                first.AddRange(new IntRange(5186, 5197));

                var Stationlists = dataPack.Station523.Where(o =>!string.IsNullOrWhiteSpace(o.Code) && first.IsCodeInRanges(o.Code) &&  o.Frame.Date == DateTime.Now.Date && o.Data.Alarm > 0).Select(o => new StockInAndOutRecordDto { dateTime = o.Frame.ToString("HH:mm:ss"), Message = o.Code + "，" + o.Data.Alarm.ToString() }).ToList();
                list.AddRange(Stationlists);

            }
            catch (Exception ex)
            {
                list.Add(new StockInAndOutRecordDto { dateTime = DateTime.Now.ToString("HH:mm:ss"), Message = $"缓存报警读取失败:{ex.Message}" });
            }

            if (list.Any())
            {
                list = list.OrderByDescending(p => p.dateTime).ToList();
            }
            return list;
        }

        #endregion

        #region 任务信息看板
        /// <summary>
        /// 时效任务信息
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [HttpPost]
        public List<dynamic> GetSxInfo(StrRequestDto request)
        {
            if (string.IsNullOrEmpty(request.StrRequest))
            {
                return new List<dynamic> { new object(), new object(), new object(), new object(), new object() };
            }
            var result = new List<dynamic>();
            if (request.StrRequest.Split(',').Contains("1"))  //堆垛机状态
            {
                //查询设备状态
                result.Add(_zhongTianSxService.GetSxEquips(new GetEquipsRequest()
                {
                    WarehouseCodeList = new List<string>() {
                     Const.Sxhouse_putong }
                }));
            }
            else
            {
                result.Add("");
            }
            if (request.StrRequest.Split(',').Contains("2")) //非控制盘信息
            {
                result.Add(_zhongTianSxService.GetCurStockInfo());

            }
            else
            {
                result.Add("");
            }
            if (request.StrRequest.Split(',').Contains("3")) //控制盘信息
            {
                result.Add(_zhongTianSxService.GetCurStockInfoControl());
            }
            else
            {
                result.Add("");
            }
            if (request.StrRequest.Split(',').Contains("4")) //报警信息
            {
                List<string> equipNoList = new List<string>();
                InOutEquipment.WorkShopWithEquipment.Where(s => s.InOrOut == "Out" && InOutEquipment.DeopWithWOrkShops.Where(m => m.Dept == request.Workshop).FirstOrDefault().WorkShopList.Contains(s.WorkShop)).ToList().ForEach(
                    n => equipNoList.AddRange(n.Equipments));
                //获取报警信息
                //设备号数据，做成配置
                result.Add(GetStockInAndOutRecordInfo(new GetEquipsRequest
                {
                    EquCodeList = equipNoList.Distinct().ToList()
                }));
            }
            else
            {
                result.Add("");
            }
            if (request.StrRequest.Split(',').Contains("5")) //库存使用率信息
            {
                result.Add(_zhongTianSxService.GetSxLocationUsageViewList());
            }
            else
            {
                result.Add("");
            }

            return result;
        }
        [HttpPost]
        public List<dynamic> GetPalletizingTaskInfo(StrRequestDto request)
        {
            if (string.IsNullOrEmpty(request.StrRequest))
            {
                return new List<dynamic> { new object() };
            }
            var result = new List<dynamic>();
            if (request.StrRequest.Split(',').Contains("1"))  //堆垛机状态
            {
                //查询设备状态
                result.Add(_zhongTianSxService.GetPalletizingInfo());
            }
            

            return result;

        }
        #endregion

        #region 包装信息看板

        #endregion

        #region 货位利用率
        /// <summary>
        /// 时效货位利用率
        /// </summary>
        /// <param name="strRequest"></param>
        /// <returns></returns>
        [HttpGet]
        public List<LocationUsageReportViewDto> GetSxLocationStateList()
        {
            return _zhongTianSxService.GetSxLocationUsageViewList();
        }
        #endregion

        #region 异常信息汇总
        /// <summary>
        /// 时效异常信息汇总
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public List<dynamic> GetTotalErrorInfo()
        {  
            var result = new List<dynamic>();
            List<string> equipNoList = new List<string>();
            InOutEquipment.WorkShopWithEquipment.Where(s => InOutEquipment.DeopWithWOrkShops.Where(m => m.Dept == "sx").FirstOrDefault().WorkShopList.Contains(s.WorkShop)).ToList().ForEach(
                n => equipNoList.AddRange(n.Equipments));
            //报警信息
            result.Add(GetStockInAndOutRecordInfo(new GetEquipsRequest()
            {
                EquCodeList = equipNoList.Distinct().ToList()
            }));
            return result;
        }
        #endregion

        #region 私有方法：将报表方法提取出来
        /// <summary>
        /// 报警信息
        /// </summary>
        /// <param name="reqEntity"></param>
        /// <returns></returns>
        private List<StockInAndOutRecordDto> GetStockInAndOutRecordInfo(GetEquipsRequest reqEntity)
        {
            var list = new List<StockInAndOutRecordDto>();
            //读取redis缓存(wcs实时传递设备信息)
            //var strMsgList = _redisClient.Get(Const.equi_list);
            //获取单据错误信息
            //List<SqlSugar.Model.BillErrorInfo> ErrorList = new List<SqlSugar.Model.BillErrorInfo>();
            //try
            //{
            //    ErrorList = _yongGuanDataService.GetBillErrorInfoList(reqEntity.EquipNo);
            //    if (ErrorList.Count() > 0)
            //    {
            //        list.AddRange(ErrorList.Select(s => new StockInAndOutRecordDto { dateTime = s.CreatedTime.ToString("HH:mm:ss"), Message = s.Msg }).ToList());
            //    }
            //    //if (ErrorList.Count() > 0)
            //    //{
            //    //    //显示近一天的数据
            //    //    //DateTime end = DateTime.Now.AddDays(1);
            //    //    ErrorList = ErrorList.Where(x => x.CreatedTime.Date == DateTime.Now.Date).ToList();
            //    //}
            //}
            //catch (Exception ex)
            //{
            //    list.Add(new StockInAndOutRecordDto { dateTime = DateTime.Now.ToString("HH:mm:ss"), Message = $"数据库报警信息读取失败:{ex.Message}" });
            //}
            try
            {
                var Len = _freeRedisClient.LLen("Packs");
                var dataPack = _freeRedisClient.LIndex<DataPack>("Packs", Len - 1);
                var Stationlists = dataPack.Station523.Where(o =>o.Frame.Date==DateTime.Now.Date &&  o.Data.Alarm > 0).Select(o =>new StockInAndOutRecordDto { dateTime=o.Frame.ToString("HH:mm:ss"), Message = o.Code + "，" + o.Data.Alarm.ToString() } ).ToList();
                //  sss.Robot522  sss.SRM542
                list.AddRange(Stationlists);
                var Robotlists = dataPack.Robot522.Where(o => o.Frame.Date == DateTime.Now.Date && o.Data.Alarm > 0).Select(o => new StockInAndOutRecordDto { dateTime = o.Frame.ToString("HH:mm:ss"), Message = o.Code + "，" + o.Data.Alarm.ToString() }).ToList();
                list.AddRange(Robotlists);
                var SRMlists = dataPack.SRM542.Where(o => o.Frame.Date == DateTime.Now.Date && o.Data.Alarm > 0).Select(o => new StockInAndOutRecordDto { dateTime = o.Frame.ToString("HH:mm:ss"), Message = o.Code + "，" + o.Data.Alarm.ToString() }).ToList();
                list.AddRange(Robotlists);

                var keylists = RedisHelper.Keys("Sx:WMSErrorInfo:**");
                foreach (var key in keylists)
                {
                    var value = RedisHelper.Get(key);
                    if (value != null)
                    {
                        var msgdto = JsonConvert.DeserializeObject<BusinessDto2>(value);
                        if (msgdto != null && !string.IsNullOrEmpty(msgdto.Con) && msgdto.Time.Date == DateTime.Now.Date)
                        {
                            list.Add(new StockInAndOutRecordDto { dateTime = msgdto.Time.ToString("HH:mm:ss"), Message = "WMS业务报警:" + msgdto.Con });

                        }
                    }
                }

                var wcskeylists = _freeRedisClient.Keys("Sx:WCSErrorInfo:**");
                foreach (var key in wcskeylists)
                {
                    var value = _freeRedisClient.Get(key);
                    if (value != null)
                    {
                        var msgdto = JsonConvert.DeserializeObject<BusinessDto2>(value);
                        if (msgdto != null && !string.IsNullOrEmpty(msgdto.Con) && msgdto.Time.Date == DateTime.Now.Date)
                        {
                            list.Add(new StockInAndOutRecordDto { dateTime = msgdto.Time.ToString("HH:mm:ss"), Message = "WCS业务报警:" + msgdto.Con });

                        }
                    }
                }         
            }
            catch (Exception ex)
            {
                list.Add(new StockInAndOutRecordDto { dateTime = DateTime.Now.ToString("HH:mm:ss"), Message = $"缓存报警读取失败:{ex.Message}" });
            }

            if (list.Any())
            {
                list = list.OrderByDescending(p => p.dateTime).ToList();
            }
            return list;
        }
        #endregion
    }
}
